import express from "express";
import Joi from 'joi';

import CriminalCase from "../../persistence/schemes/CriminalCase";
import CriminalCaseDAO, {Get_collection_name} from '../../persistence/DAOs/CriminalCaseDAO';
import Validator, {ValidationSource} from "./Validator";

export {Get_collection_name};

const criminal_case_id = Joi.object().keys({
    criminal_case_number: Joi.string().required().max(10),
    jurisdiction_name: Joi.string().required().max(30)
});
const criminal_case_Joi_schema = Joi.object().keys({
    criminal_case_number: Joi.string().required().max(10),
    jurisdiction_name: Joi.string().required().max(30),
    date_of_criminal_case: Joi.date().required()
});

// Mini. app. for '/criminal_cases' sub-path:
export const router = express.Router();
// http://localhost:1963/NYCP/API/criminal_cases
router.get('', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    response.redirect(request.baseUrl + request.path + 'all');
});
// http://localhost:1963/NYCP/API/criminal_cases/one?criminal_case_number=M13000&jurisdiction_name=Marseille
router.get('/one', Validator(criminal_case_id /*, ValidationSource.QUERY */), (request, response) => {
    console.log("READ: " + request.baseUrl + request.path + JSON.stringify(request.query));
    CriminalCaseDAO.GetCriminalCase(request.query.criminal_case_number as string,
        request.query.jurisdiction_name as string).then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// http://localhost:1963/NYCP/API/criminal_cases/all
router.get('/all', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    CriminalCaseDAO.GetCriminalCases().then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// curl -d "criminal_case_number=P64000&jurisdiction_name=Pau&date_of_criminal_case=2002-03-23" http://localhost:1963/NYCP/API/criminal_cases/
router.post('/', Validator(criminal_case_Joi_schema, ValidationSource.BODY),
    async (request, response) => {
        console.log("CREATE: " + request.baseUrl + request.path + JSON.stringify(request.body));
        try {
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.body.criminal_case_number, request.body.jurisdiction_name);
            if (criminal_case) throw new Error('Criminal case ' + JSON.stringify(request.body) + ' already exists...');
            CriminalCaseDAO.Create(request.body as CriminalCase).then(data => response.status(201).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X PUT http://localhost:1963/NYCP/API/criminal_cases/P64000\&Pau\&2002-03-23
router.put('/:criminal_case_number&:jurisdiction_name&:date_of_criminal_case', Validator(criminal_case_Joi_schema, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("UPDATE: " + request.baseUrl + request.path + JSON.stringify(request.params));
        try {
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.params.criminal_case_number, request.params.jurisdiction_name);
            if (!criminal_case) throw new Error('Criminal case ' + request.params.criminal_case_number + '-' + request.params.jurisdiction_name + ' does not exist...');
            criminal_case.date_of_criminal_case = new Date(request.params.date_of_criminal_case);
            CriminalCaseDAO.Update(criminal_case)
                .then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X DELETE http://localhost:1963/NYCP/API/criminal_cases/P64000\&Pau
router.delete('/:criminal_case_number&:jurisdiction_name', Validator(criminal_case_id, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("DELETE: " + request.baseUrl + request.path + JSON.stringify(request.params));
        try {
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.params.criminal_case_number, request.params.jurisdiction_name);
            if (!criminal_case) throw new Error('Criminal case ' + request.params.criminal_case_number + '-' + request.params.jurisdiction_name + ' does not exist...');
            CriminalCaseDAO.Delete(criminal_case)
                .then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
