import express from 'express';
import Joi from 'joi';
import * as _ from 'lodash';

import JudicialDecision from "../../persistence/schemes/JudicialDecision";
import JudicialDecisionDAO, {Get_collection_name} from '../../persistence/DAOs/JudicialDecisionDAO';
import PrisonerDAO from "../../persistence/DAOs/PrisonerDAO";
import Validator, {ValidationSource} from "./Validator";

export {Get_collection_name};

const judicial_decision_id = Joi.object().keys({
    decision_type_number: Joi.string().required().valid('1', '2', '3').max(1),
    prison_file_number: Joi.string().required().max(10),
    date_of_decision: Joi.date().required()
});
const judicial_decision_Joi_schema = Joi.object().keys({
    decision_type_number: Joi.string().required().valid('1', '2', '3').max(1),
    prison_file_number: Joi.string().required().max(10),
    date_of_decision: Joi.date().required(),
    date_of_final_discharge: Joi.date().optional(),
    duration: Joi.number().optional().integer().greater(0)
}).xor('date_of_final_discharge', 'duration');

// Mini. app. for '/judicial_decisions' sub-path:
export const router = express.Router();
// http://localhost:1963/NYCP/API/judicial_decisions
router.get('', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    response.redirect(request.baseUrl + request.path + 'all');
});
// http://localhost:1963/NYCP/API/judicial_decisions/one?decision_type_number=1&prison_file_number=1993&date_of_decision=2002-03-23
router.get('/one', Validator(judicial_decision_id /*, ValidationSource.QUERY*/),
    async (request, response) => {
        console.log("READ: " + request.baseUrl + request.path + JSON.stringify(request.query));
        // Get prisoner id. with 'true':
        const prisoner = await PrisonerDAO.GetPrisoner(request.body.prison_file_number, true);
        if (!prisoner) throw new Error('Prisoner ' + request.body.prison_file_number + ' does not exist...');
        JudicialDecisionDAO.GetJudicialDecision(request.query.decision_type_number as string,
            prisoner._id,
            new Date(request.query.date_of_decision as string)).then(data => response.status(200).json(data))
            .catch(error => response.status(400).json({error, message: error.message}));
    });
// http://localhost:1963/NYCP/API/judicial_decisions/all
router.get('/all', (request, res) => {
    console.log("READ: " + request.baseUrl + request.path);
    JudicialDecisionDAO.GetJudicialDecisions().then(data => res.status(200).json(data))
        .catch(error => res.status(400).json({error, message: error.message}));
});
// curl -d "decision_type_number=1&prison_file_number=1993&date_of_decision=2002-03-23&duration=100" http://localhost:1963/NYCP/API/judicial_decisions/
router.post('/', Validator(judicial_decision_Joi_schema, ValidationSource.BODY),
    async (request, response) => {
        console.log("CREATE: " + request.baseUrl + request.path + JSON.stringify(request.body));
        try {
            // Get prisoner id. with 'true':
            const prisoner = await PrisonerDAO.GetPrisoner(request.body.prison_file_number, true);
            if (!prisoner) throw new Error('Prisoner ' + request.body.prison_file_number + ' does not exist...');
            const judicial_decision = await JudicialDecisionDAO.GetJudicialDecision(request.body.decision_type_number,
                prisoner._id,
                new Date(request.body.date_of_decision));
            if (judicial_decision) throw new Error('Judicial decision ' + JSON.stringify(request.body) + ' already exists...');
            JudicialDecisionDAO.Create(
                _.omit(request.body, ['prison_file_number']) as JudicialDecision, prisoner._id).then(data => response.status(201).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X PUT http://localhost:1963/NYCP/API/judicial_decisions/1\&1993\&2002-03-23\&200
// router.put('/:decision_type_number&:prison_file_number&:date_of_decision&:duration', Validator(judicial_decision_id, ValidationSource.PARAMS),
//     async (request, res) => {
//         console.log("UPDATE: " + request.baseUrl + request.path + JSON.stringify(request.params));
//         try {
//             // Get prisoner id. with 'true':
//             const prisoner = await PrisonerDAO.GetPrisoner(request.params.prison_file_number, true);
//             if (!prisoner) throw new Error('Prisoner ' + request.params.prison_file_number + ' does not exist...');
//             const judicial_decision = await JudicialDecisionDAO.GetJudicialDecision(request.params.decision_type_number,
//                 prisoner._id,
//                 new Date(request.params.date_of_decision));
//             if (!judicial_decision) throw new Error('Judicial decision ' + JSON.stringify(request.params) + ' does not exist...');
//             judicial_decision.duration = request.params.duration;
//             JudicialDecisionDAO.Update(judicial_decision).then(data => res.status(200).json(data))
//                 .catch(error => res.status(400).json({error, message: error.message}));
//         } catch (error) {
//             res.status(400).json({error, message: error.message});
//         }
//     });
// curl -X DELETE http://localhost:1963/NYCP/API/judicial_decisions/1\&1993\&2002-03-23
router.delete('/:decision_type_number&:prison_file_number&:date_of_decision', Validator(judicial_decision_id, ValidationSource.PARAMS),
    async (request, response) => {
        try {
            console.log("DELETE: " + request.baseUrl + request.path + JSON.stringify(request.params));
            // Get prisoner id. with 'true':
            const prisoner = await PrisonerDAO.GetPrisoner(request.params.prison_file_number, true);
            if (!prisoner) throw new Error('Prisoner ' + request.params.prison_file_number + ' does not exist...');
            const judicial_decision = await JudicialDecisionDAO.GetJudicialDecision(request.params.decision_type_number,
                prisoner._id,
                new Date(request.params.date_of_decision));
            if (!judicial_decision) throw new Error('Judicial decision ' + JSON.stringify(request.params) + ' does not exist...');
            JudicialDecisionDAO.Delete(judicial_decision).then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
