import express from "express";
import Joi from 'joi';

import Motive from "../../persistence/schemes/Motive";
import MotiveDAO, {Get_collection_name} from "../../persistence/DAOs/MotiveDAO";
import Validator, {ValidationSource} from "./Validator";

export {Get_collection_name};

// https://stackoverflow.com/questions/58811509/what-is-the-difference-between-joi-object-and-joi-object-keys
export const motive_id = Joi.object().keys({ // 'keys' is useful when Joi schema is modified at different places...
    motive_number: Joi.string().required().max(10),
});
const motive_Joi_schema = Joi.object().keys({
    motive_number: Joi.string().required().max(10),
    motive_label: Joi.string().required().max(50)
});

// Mini. app. for '/motives' sub-path:
export const router = express.Router();
// http://localhost:1963/NYCP/API/motives
router.get('', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    response.redirect(request.baseUrl + request.path + 'all');
});
// http://localhost:1963/NYCP/API/motives/one?motive_number=01
/**
 * 'Validator(motive_id, ValidationSource.PARAMS)' is passed as first handler;
 * it stops 'get' processing if Joi schema validation fails...
 */
router.get('/one', Validator(motive_id /*, ValidationSource.QUERY */), (request, response) => {
    console.log("READ: " + request.baseUrl + request.path + JSON.stringify(request.query));
    MotiveDAO.GetMotive(request.query.motive_number as string).then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// http://localhost:1963/NYCP/API/motives/all
router.get('/all', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    MotiveDAO.GetMotives().then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// curl -d "motive_number=15&motive_label=procuring for prostitutes" http://localhost:1963/NYCP/API/motives/
router.post('/', Validator(motive_Joi_schema, ValidationSource.BODY),
    async (request, response) => {
        console.log("CREATE: " + request.baseUrl + request.path);
        try {
            const motive = await MotiveDAO.GetMotive(request.body.motive_number);
            if (motive) throw new Error('Motive ' + request.body.motive_number + ' already exists...');
            MotiveDAO.Create({
                motive_number: request.body.motive_number,
                motive_label: request.body.motive_label
            } as Motive).then(data => response.status(201).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    }
);
// curl -X PUT http://localhost:1963/NYCP/API/motives/01\&various%20misdemeanors%20and%20theft
// curl -X PUT http://localhost:1963/NYCP/API/motives/01\&theft%20and%20various%20misdemeanors
router.put('/:motive_number&:motive_label', Validator(motive_Joi_schema, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("UPDATE: " + request.baseUrl + request.path);
        try {
            const motive = await MotiveDAO.GetMotive(request.params.motive_number)
            if (!motive) throw new Error('Motive ' + request.params.motive_number + ' does not exist...');
            motive.motive_label = request.params.motive_label;
            MotiveDAO.Update(motive)
                .then(data => response.status(200).json({data}))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X DELETE http://localhost:1963/NYCP/API/motives/15
router.delete('/:motive_number', Validator(motive_id, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("DELETE: " + request.baseUrl + request.path);
        try {
            const motive = await MotiveDAO.GetMotive(request.params.motive_number)
            if (!motive) throw new Error('Motive ' + request.params.motive_number + ' does not exist...');
            MotiveDAO.Delete(motive)
                .then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    }
);
