import express from "express";
import Joi from 'joi';

/** Create-Delete links between 'Prisoner' and 'CriminalCase' by means of the 'offense' navigation in 'Prisoner' */

import CriminalCaseDAO from "../../persistence/DAOs/CriminalCaseDAO";
import OffenseDAO, {Get_collection_name} from "../../persistence/DAOs/OffenseDAO";
import PrisonerDAO from "../../persistence/DAOs/PrisonerDAO";
import Validator, {ValidationSource} from "./Validator";

export {Get_collection_name};

const prisoner_id = Joi.object().keys({
    prison_file_number: Joi.string().required().max(10),
});
const offense_Joi_schema = Joi.object().keys({
    prison_file_number: Joi.string().required().max(10),
    criminal_case_number: Joi.string().required().max(10),
    jurisdiction_name: Joi.string().required().max(30)
});
// Mini. app. for '/offenses' sub-path:
export const router = express.Router();
// curl http://localhost:1963/NYCP/API/offenses/all?prison_file_number=2001
router.get('/all', Validator(prisoner_id /*, ValidationSource.QUERY */),
    async (request, response) => {
        console.log("READ: " + request.baseUrl + request.path + JSON.stringify(request.query));
        const prisoner = await PrisonerDAO.GetPrisoner(request.query.prison_file_number as string, true);
        if (!prisoner) throw new Error('Prisoner ' + request.query.prison_file_number + ' does not exist...');
        OffenseDAO.GetOffenses(prisoner._id).then(data => response.status(200).json(data))
            .catch(error => response.status(400).json({error, message: error.message}));
    });
// curl -d "prison_file_number=2001&criminal_case_number=M13000&jurisdiction_name=Marseille" http://localhost:1963/NYCP/API/offenses/
router.post('/', Validator(offense_Joi_schema, ValidationSource.BODY),
    async (request, response) => {
        try {
            console.log("CREATE: " + request.baseUrl + request.path + JSON.stringify(request.body));
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.body.criminal_case_number, request.body.jurisdiction_name, true);
            if (!criminal_case) throw new Error('Criminal case ' + request.body.criminal_case_number + '-' + request.body.jurisdiction_name + ' does not exist...');
            const prisoner = await PrisonerDAO.GetPrisoner(request.body.prison_file_number, true);
            if (!prisoner) throw new Error('Prisoner ' + request.body.prison_file_number + ' does not exist...');
            OffenseDAO.Create(criminal_case._id, prisoner._id).then(data => response.status(201).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X DELETE http://localhost:1963/NYCP/API/offenses/2001\&N44000\&Nantes
router.delete('/:prison_file_number&:criminal_case_number&:jurisdiction_name', Validator(offense_Joi_schema, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("DELETE: " + request.baseUrl + request.path + JSON.stringify(request.params));
        try {
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.params.criminal_case_number, request.params.jurisdiction_name, true);
            if (!criminal_case) throw new Error('Criminal case ' + request.params.criminal_case_number + '-' + request.params.jurisdiction_name + ' does not exist...');
            const prisoner = await PrisonerDAO.GetPrisoner(request.params.prison_file_number, true);
            if (!prisoner) throw new Error('Prisoner ' + request.params.prison_file_number + ' does not exist...');
            OffenseDAO.Delete(criminal_case._id, prisoner._id)
                .then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
