import express from 'express';
import Joi from 'joi';
import * as _ from 'lodash';

import CriminalCase from "../../persistence/schemes/CriminalCase";
import CriminalCaseDAO from "../../persistence/DAOs/CriminalCaseDAO";
import Motive from "../../persistence/schemes/Motive";
import MotiveDAO from "../../persistence/DAOs/MotiveDAO";
import Prisoner from "../../persistence/schemes/Prisoner";
import PrisonerDAO, {Get_collection_name} from "../../persistence/DAOs/PrisonerDAO";
import Validator, {ValidationSource} from "./Validator";

export {Get_collection_name};

const prisoner_id = Joi.object().keys({ // 'keys' is useful when Joi schema is modified at different places...
    prison_file_number: Joi.string().required().max(10),
});
const prisoner_Joi_schema = Joi.object().keys({
    prison_file_number: Joi.string().required().max(10),
    given_name: Joi.string()/*.required()*/.max(30).trim(),
    surname: Joi.string()/*.required()*/.max(30).trim(),
    date_of_birth: Joi.date()/*.required()*/,
    place_of_birth: Joi.string()/*.required()*/.max(30).trim(),
    date_of_incarceration: Joi.date().required(),
    criminal_case_number: Joi.string().required().max(10),
    jurisdiction_name: Joi.string().required().max(30),
    date_of_criminal_case: Joi.date()/*.required()*/,
    motive_number: Joi.string().required().max(10),
    motive_label: Joi.string()/*.required()*/.max(50)
});

// Mini. app. for '/prisoners' sub-path:
export const router = express.Router();
// http://localhost:1963/NYCP/API/prisoners
router.get('', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    response.redirect(request.baseUrl + request.path + 'all');
});
// http://localhost:1963/NYCP/API/prisoners/one?prison_file_number=1993
router.get('/one', Validator(prisoner_id /*, ValidationSource.QUERY */), (request, response) => {
    console.log("READ: " + request.baseUrl + request.path + JSON.stringify(request.query));
    PrisonerDAO.GetPrisoner(request.query.prison_file_number as string).then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// http://localhost:1963/NYCP/API/prisoners/all
router.get('/all', (request, response) => {
    console.log("READ: " + request.baseUrl + request.path);
    PrisonerDAO.GetPrisoners().then(data => response.status(200).json(data))
        .catch(error => response.status(400).json({error, message: error.message}));
});
// curl -d "prison_file_number=1993&date_of_incarceration=1993-12-06&criminal_case_number=M13000&jurisdiction_name=Marseille&motive_number=01" http://localhost:1963/NYCP/API/prisoners/
// PowerShell:
// $create_1993 = @{prison_file_number='1993';date_of_incarceration='1993-12-06';criminal_case_number='M13000';jurisdiction_name='Marseille';motive_number='01'}
// Invoke-WebRequest -Uri http://localhost:1963/NYCP/API/prisoners/ -Method POST -Body $create_1993

// Criminal case does not yet exist:
// curl -d "prison_file_number=1996&date_of_incarceration=1996-04-15&criminal_case_number=N44000&jurisdiction_name=Nantes&date_of_criminal_case=1996-04-15&motive_number=03" http://localhost:1963/NYCP/API/prisoners/
// Motive does not yet exist:
// curl -d "prison_file_number=2001&date_of_incarceration=2001-02-22&criminal_case_number=L69000&jurisdiction_name=Lyon&motive_number=05&motive_label=drunk driving" http://localhost:1963/NYCP/API/prisoners/
router.post('/', Validator(prisoner_Joi_schema, ValidationSource.BODY),
    async (request, response) => {
        try {
            console.log("CREATE: " + request.baseUrl + request.path + JSON.stringify(request.body));
            let criminal_case = await CriminalCaseDAO.GetCriminalCase(request.body.criminal_case_number, request.body.jurisdiction_name, true);
            if (!criminal_case)
                criminal_case = await CriminalCaseDAO.Create({
                    criminal_case_number: request.body.criminal_case_number,
                    jurisdiction_name: request.body.jurisdiction_name,
                    date_of_criminal_case: request.body.date_of_criminal_case
                } as CriminalCase);
            let motive = await MotiveDAO.GetMotive(request.body.motive_number, true);
            if (!motive)
                motive = await MotiveDAO.Create({
                    motive_number: request.body.motive_number,
                    motive_label: request.body.motive_label
                } as Motive);
            const prisoner = await PrisonerDAO.GetPrisoner(request.body.prison_file_number, true);
            if (prisoner) throw new Error('Prisoner ' + request.body.prison_file_number + ' already exists...');
            PrisonerDAO.Create(_.omit(request.body, ['criminal_case_number',
                'jurisdiction_name',
                'date_of_criminal_case',
                'motive_number',
                'motive_label']) as Prisoner, criminal_case._id, motive._id).then(data => response.status(201).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X PUT http://localhost:1963/NYCP/API/prisoners/2001\&2002-03-23\&M13000\&Marseille\&02
router.put('/:prison_file_number&:date_of_incarceration&:criminal_case_number&:jurisdiction_name&:motive_number', Validator(prisoner_Joi_schema, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("UPDATE: " + request.baseUrl + request.path + JSON.stringify(request.params));
        try {
            const criminal_case = await CriminalCaseDAO.GetCriminalCase(request.params.criminal_case_number, request.params.jurisdiction_name, true);
            if (!criminal_case) throw new Error('Criminal case ' + request.params.criminal_case_number + '-' + request.params.jurisdiction_name + ' does not exist...');
            const motive = await MotiveDAO.GetMotive(request.params.motive_number, true)
            if (!motive) throw new Error('Motive ' + request.params.motive_number + ' does not exist...');
            const prisoner = await PrisonerDAO.GetPrisoner(request.params.prison_file_number);
            if (!prisoner) throw new Error('Prisoner ' + request.params.prison_file_number + ' does not exist...');
            prisoner.date_of_incarceration = new Date(request.params.date_of_incarceration);
            PrisonerDAO.Update(prisoner as Prisoner, criminal_case._id, motive._id).then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
// curl -X DELETE http://localhost:1963/NYCP/API/prisoners/1993
router.delete('/:prison_file_number', Validator(prisoner_id, ValidationSource.PARAMS),
    async (request, response) => {
        console.log("DELETE: " + request.baseUrl + request.path + JSON.stringify(request.params));
        try {
            const prisoner = await PrisonerDAO.GetPrisoner(request.params.prison_file_number);
            if (!prisoner) throw new Error('Prisoner ' + request.params.prison_file_number + ' does not exist...');
            PrisonerDAO.Delete(prisoner).then(data => response.status(200).json(data))
                .catch(error => response.status(400).json({error, message: error.message}));
        } catch (error: unknown) {
            response.status(400).json({error, message: (error as Error).message});
        }
    });
